@@ -104,6 +104,40 @@ static char *command_file = NULL;
 static int debug = 0;
 static int nostdio = 0;
 
+/*
+ * Keep track of open sockets so we can close them on exec().
+ */
+typedef struct isim_fd {
+    int fd;
+    struct isim_fd *next;
+} isim_fd_t;
+
+static isim_fd_t *isim_fds = NULL;
+
+static void isim_add_fd(int fd)
+{
+    isim_fd_t *n = malloc(sizeof(*n));
+
+    if (!n) {
+	fprintf(stderr, "Unable to add fd to list, out of memory\n");
+	exit(1);
+    }
+
+    n->fd = fd;
+    n->next = isim_fds;
+    isim_fds = n;
+}
+
+static void isim_close_fds(void)
+{
+    isim_fd_t *n = isim_fds;
+
+    while(n) {
+	close(n->fd);
+	n = n->next;
+    }
+}
+
 static void shutdown_handler(int sig);
 
 typedef struct misc_data misc_data_t;
@@ -135,6 +169,8 @@ struct misc_data
     console_info_t *consoles;
 };
 
+static misc_data_t *global_misc_data;
+
 static void *
 balloc(sys_data_t *sys, int size)
 {
@@ -213,13 +249,11 @@ lan_send(lanserv_data_t *lan,
     if (!l)
 	return;
 
+    memset(&msg, 0, sizeof(msg));
     msg.msg_name = &(l->addr);
     msg.msg_namelen = l->addr_len;
     msg.msg_iov = data;
     msg.msg_iovlen = vecs;
-    msg.msg_control = NULL;
-    msg.msg_controllen = 0;
-    msg.msg_flags = 0;
 
     rv = sendmsg(l->xmit_fd, &msg, 0);
     if (rv) {
@@ -302,6 +336,8 @@ open_lan_fd(struct sockaddr *addr, socklen_t addr_len)
 	exit(1);
     }
 
+    isim_add_fd(fd);
+
     return fd;
 }
 
@@ -332,7 +368,7 @@ lan_channel_init(void *info, channel_t *chan)
     }
 
     if (lan->lan_addr_set) {
-	lan_fd = open_lan_fd(&lan->lan_addr.addr.s_ipsock.s_addr,
+	lan_fd = open_lan_fd(&lan->lan_addr.addr.s_ipsock.s_addr0,
 			     lan->lan_addr.addr_len);
 	if (lan_fd == -1) {
 	    fprintf(stderr, "Unable to open LAN address\n");
@@ -443,8 +479,7 @@ ser_channel_init(void *info, channel_t *chan)
     serserv_data_t *ser = chan->chan_info;
     int err;
     int fd;
-    int opt;
-    struct sockaddr *addr = &ser->addr.addr.s_ipsock.s_addr;
+    struct sockaddr *addr = &ser->addr.addr.s_ipsock.s_addr0;
     os_hnd_fd_id_t *fd_id;
     int val;
 
@@ -526,12 +561,15 @@ ser_channel_init(void *info, channel_t *chan)
 	}
     }
 
+    if (!err)
+	isim_add_fd(fd);
+
     return err;
 }
 
 static void
-isim_log(sys_data_t *sys, int logtype, msg_t *msg, char *format, va_list ap,
-	 int len)
+isim_log(sys_data_t *sys, int logtype, msg_t *msg, const char *format,
+	 va_list ap, int len)
 {
     misc_data_t *data = sys->info;
     char *str;
@@ -583,7 +621,7 @@ isim_log(sys_data_t *sys, int logtype, msg_t *msg, char *format, va_list ap,
 }
 
 static void
-sim_log(sys_data_t *sys, int logtype, msg_t *msg, char *format, ...)
+sim_log(sys_data_t *sys, int logtype, msg_t *msg, const char *format, ...)
 {
     va_list ap;
     char dummy;
@@ -598,7 +636,7 @@ sim_log(sys_data_t *sys, int logtype, msg_t *msg, char *format, ...)
 }
 
 static void
-sim_chan_log(channel_t *chan, int logtype, msg_t *msg, char *format, ...)
+sim_chan_log(channel_t *chan, int logtype, msg_t *msg, const char *format, ...)
 {
     va_list ap;
     char dummy;
@@ -608,7 +646,7 @@ sim_chan_log(channel_t *chan, int logtype, msg_t *msg, char *format, ...)
     len = vsnprintf(&dummy, 1, format, ap);
     va_end(ap);
     va_start(ap, format);
-    isim_log(NULL, logtype, msg, format, ap, len);
+    isim_log(global_misc_data->sys, logtype, msg, format, ap, len);
     va_end(ap);
 }
 
@@ -692,7 +730,6 @@ emu_printf(emu_out_t *out, char *format, ...)
 {
     console_info_t *info = out->data;
     va_list ap;
-    int rv;
     char buffer[500];
     int start = 0;
     int pos;
@@ -702,13 +739,13 @@ emu_printf(emu_out_t *out, char *format, ...)
     va_end(ap);
     for (pos = 0; buffer[pos]; pos++) {
 	if (buffer[pos] == '\n') {
-	    rv = write(info->outfd, buffer + start, pos - start + 1);
-	    rv = write(info->outfd, "\r", 1);
+	    (void) write(info->outfd, buffer + start, pos - start + 1);
+	    (void) write(info->outfd, "\r", 1);
 	    start = pos + 1;
 	}
     }
     if (pos != start)
-	rv = write(info->outfd, buffer + start, pos - start);
+	(void) write(info->outfd, buffer + start, pos - start);
 }
 
 static void
@@ -727,8 +764,6 @@ dummy_printf(emu_out_t *out, char *format, ...)
 static unsigned char
 handle_telnet(console_info_t *info, unsigned char c)
 {
-    int err;
-
     info->tn_buf[info->tn_pos++] = c;
     if ((info->tn_pos == 2) && (info->tn_buf[1] == TN_IAC))
 	/* Double IAC, just send it on. */
@@ -767,12 +802,12 @@ handle_telnet(console_info_t *info, unsigned char c)
 
  send_wont:
     info->tn_buf[1] = TN_WONT;
-    err = write(info->outfd, info->tn_buf, 3);
+    (void) write(info->outfd, info->tn_buf, 3);
     goto cmd_done;
 
  send_dont:
     info->tn_buf[1] = TN_DONT;
-    err = write(info->outfd, info->tn_buf, 3);
+    (void) write(info->outfd, info->tn_buf, 3);
     goto cmd_done;
 
  cmd_done:
@@ -783,8 +818,6 @@ handle_telnet(console_info_t *info, unsigned char c)
 static int
 handle_user_char(console_info_t *info, unsigned char c)
 {
-    int rv;
-
     if (info->tn_pos)
 	c = handle_telnet(info, c);
 
@@ -805,14 +838,14 @@ handle_user_char(console_info_t *info, unsigned char c)
 	if (info->pos > 0) {
 	    info->pos--;
 	    if (info->echo)
-		rv = write(info->outfd, "\b \b", 3);
+		(void) write(info->outfd, "\b \b", 3);
 	}
 	break;
 
     case 4:
 	if (info->pos == 0) {
 	    if (info->echo)
-		rv = write(info->outfd, "\n", 1);
+		(void) write(info->outfd, "\n", 1);
 	    return 1;
 	}
 	break;
@@ -820,9 +853,9 @@ handle_user_char(console_info_t *info, unsigned char c)
     case 10:
     case 13:
 	if (info->echo) {
-	    rv = write(info->outfd, "\n", 1);
+	    (void) write(info->outfd, "\n", 1);
 	    if (info->telnet)
-		rv = write(info->outfd, "\r", 1);
+		(void) write(info->outfd, "\r", 1);
 	}
 	info->buffer[info->pos] = '\0';
 	if (strcmp(info->buffer, "noecho") == 0) {
@@ -830,7 +863,7 @@ handle_user_char(console_info_t *info, unsigned char c)
 	} else {
 	    ipmi_emu_cmd(&info->out, info->data->emu, info->buffer);
 	}
-	rv = write(info->outfd, "> ", 2);
+	(void) write(info->outfd, "> ", 2);
 	info->pos = 0;
 	break;
 
@@ -838,12 +871,12 @@ handle_user_char(console_info_t *info, unsigned char c)
     default:
 	if (info->pos >= sizeof(info->buffer)-1) {
 	    char *msg = "\nCommand is too long, max of %d characters\n";
-	    rv = write(info->outfd, msg, strlen(msg));
+	    (void) write(info->outfd, msg, strlen(msg));
 	} else {
 	    info->buffer[info->pos] = c;
 	    info->pos++;
 	    if (info->echo)
-		rv = write(info->outfd, &c, 1);
+		(void) write(info->outfd, &c, 1);
 	}
     }
 
@@ -937,6 +970,7 @@ console_bind_ready(int fd, void *cb_data, os_hnd_fd_id_t *id)
 	err = write(rv, msg, strlen(msg));
 	close(rv);
 	free(newcon);
+	return;
     }
 
     newcon->next = misc->consoles;
@@ -962,6 +996,8 @@ init_term(void)
     new_termios.c_iflag &= ~(IGNBRK|BRKINT|PARMRK|ISTRIP
 			     |INLCR|IGNCR|ICRNL|IXON);
     new_termios.c_lflag &= ~(ECHO|ECHONL|ICANON|ISIG|IEXTEN);
+    new_termios.c_cc[VTIME] = 0;
+    new_termios.c_cc[VMIN] = 0;
     tcsetattr(0, TCSADRAIN, &new_termios);
 }
 
@@ -1201,9 +1237,8 @@ static void
 handle_sigchld(int sig)
 {
     unsigned char c = 1;
-    int rv;
 
-    rv = write(sigpipeh[1], &c, 1);
+    (void) write(sigpipeh[1], &c, 1);
 }
 
 static ipmi_child_quit_t *child_quit_handlers;
@@ -1283,6 +1318,7 @@ ipmi_do_start_cmd(startcmd_t *startcmd)
     if (pid == 0) {
 	char *args[4] = { "/bin/sh", "-c", cmd, NULL };
 
+	isim_close_fds();
 	execvp(args[0], args);
 	exit(1);
     }
@@ -1346,6 +1382,8 @@ main(int argc, const char *argv[])
 
     printf("IPMI Simulator version %s\n", PVERSION);
 
+    global_misc_data = &data;
+
     data.os_hnd = ipmi_posix_setup_os_handler();
     if (!data.os_hnd) {
 	fprintf(stderr, "Unable to allocate OS handler\n");
@@ -1523,7 +1561,7 @@ main(int argc, const char *argv[])
 	int nfd;
 	int val;
 
-	nfd = socket(sysinfo.console_addr.s_ipsock.s_addr.sa_family,
+	nfd = socket(sysinfo.console_addr.s_ipsock.s_addr0.sa_family,
 		     SOCK_STREAM, IPPROTO_TCP);
 	if (nfd == -1) {
 	    perror("Console socket open");
@@ -1555,6 +1593,8 @@ main(int argc, const char *argv[])
 	if (err) {
 	    fprintf(stderr, "Unable to add console wait: 0x%x\n", err);
 	    goto out;
+	} else {
+	    isim_add_fd(nfd);
 	}
     }
 
