@@ -66,7 +66,7 @@ extern void posix_vlog(char                 *format,
 
 typedef struct pt_os_hnd_data_s
 {
-    selector_t       *sel;
+    struct selector_s *sel;
     os_vlog_t        log_handler;
     int              wake_sig;
     struct sigaction oldact;
@@ -120,7 +120,7 @@ add_fd(os_handler_t       *handler,
     os_hnd_fd_id_t   *fd_data;
     int              rv;
     pt_os_hnd_data_t *info = handler->internal_data;
-    selector_t       *posix_sel = info->sel;
+    struct selector_s *posix_sel = info->sel;
 
     fd_data = malloc(sizeof(*fd_data));
     if (!fd_data)
@@ -149,7 +149,7 @@ static int
 remove_fd(os_handler_t *handler, os_hnd_fd_id_t *fd_data)
 {
     pt_os_hnd_data_t *info = handler->internal_data;
-    selector_t       *posix_sel = info->sel;
+    struct selector_s *posix_sel = info->sel;
 
     sel_set_fd_read_handler(posix_sel, fd_data->fd, SEL_FD_HANDLER_DISABLED);
     sel_clear_fd_handlers(posix_sel, fd_data->fd);
@@ -169,7 +169,7 @@ struct os_hnd_timer_id_s
 };
 
 static void
-timer_handler(selector_t  *sel,
+timer_handler(struct selector_s *sel,
 	      sel_timer_t *timer,
 	      void        *data)
 {
@@ -247,7 +247,7 @@ alloc_timer(os_handler_t      *handler,
     os_hnd_timer_id_t *timer_data;
     int               rv;
     pt_os_hnd_data_t  *info = handler->internal_data;
-    selector_t        *posix_sel = info->sel;
+    struct selector_s *posix_sel = info->sel;
 
     timer_data = malloc(sizeof(*timer_data));
     if (!timer_data)
@@ -392,11 +392,6 @@ sposix_log(os_handler_t         *handler,
 struct os_hnd_lock_s
 {
     pthread_mutex_t mutex;
-    int             lock_count;
-
-    /* This is volatile and we always set the owner before we set the count.
-       That avoids race conditions checking the count and owner. */
-    volatile pthread_t owner;
 };
 
 static int
@@ -404,19 +399,30 @@ create_lock(os_handler_t  *handler,
 	    os_hnd_lock_t **id)
 {
     os_hnd_lock_t *lock;
+    pthread_mutexattr_t attr;
     int           rv;
 
     lock = malloc(sizeof(*lock));
     if (!lock)
 	return ENOMEM;
-    rv = pthread_mutex_init(&lock->mutex, NULL);
-    if (rv) {
-	free(lock);
-	return rv;
-    }
-    lock->lock_count = 0;
+    rv = pthread_mutexattr_init(&attr);
+    if (rv)
+	goto out_err;
+    rv = pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_RECURSIVE);
+    if (rv)
+	goto out_err_destroy;
+    rv = pthread_mutex_init(&lock->mutex, &attr);
+    if (rv)
+	goto out_err_destroy;
+    pthread_mutexattr_destroy(&attr);
     *id = lock;
     return 0;
+
+ out_err_destroy:
+    pthread_mutexattr_destroy(&attr);
+ out_err:
+    free(lock);
+    return rv;
 }
 
 static int
@@ -425,9 +431,6 @@ destroy_lock(os_handler_t  *handler,
 {
     int rv;
 
-    if (id->lock_count != 0)
-	handler->log(handler, IPMI_LOG_FATAL,
-		     "Destroy of lock when count is not zero");
     rv = pthread_mutex_destroy(&id->mutex);
     if (rv)
 	return rv;
@@ -441,13 +444,9 @@ lock(os_handler_t  *handler,
 {
     int rv;
 
-    if ((id->lock_count == 0) || (pthread_self() != id->owner)) {
-	rv = pthread_mutex_lock(&id->mutex);
-	if (rv)
-	    return rv;
-    }
-    id->owner = pthread_self();
-    id->lock_count++;
+    rv = pthread_mutex_lock(&id->mutex);
+    if (rv)
+	return rv;
     return 0;
 }
 
@@ -457,18 +456,9 @@ unlock(os_handler_t  *handler,
 {
     int rv;
 
-    if (id->lock_count == 0)
-	handler->log(handler, IPMI_LOG_FATAL, "lock count went negative");
-    if (pthread_self() != id->owner)
-	handler->log(handler, IPMI_LOG_FATAL, "lock release by non-owner");
-    id->lock_count--;
-    if (id->lock_count == 0) {
-	rv = pthread_mutex_unlock(&id->mutex);
-	if (rv) {
-	    id->lock_count++;
-	    return rv;
-	}
-    }
+    rv = pthread_mutex_unlock(&id->mutex);
+    if (rv)
+	return rv;
     return 0;
 }
 
@@ -531,15 +521,8 @@ cond_wait(os_handler_t  *handler,
 	  os_hnd_lock_t *lock)
 {
     int       rv;
-    int       old_lock_count;
-    pthread_t old_owner;
 
-    old_lock_count = lock->lock_count; 
-    old_owner = lock->owner;
-    lock->lock_count = 0;
     rv = pthread_cond_wait(&cond->cond, &lock->mutex);
-    lock->lock_count = old_lock_count;
-    lock->owner = old_owner;
     return rv;
 }
 
@@ -552,8 +535,6 @@ cond_timedwait(os_handler_t   *handler,
     struct timespec spec;
     struct timeval  now;
     int             rv;
-    int             old_lock_count;
-    pthread_t       old_owner;
 
     rv = handler->get_monotonic_time(handler, &now);
     if (rv)
@@ -565,12 +546,7 @@ cond_timedwait(os_handler_t   *handler,
 	spec.tv_sec += 1;
 	spec.tv_nsec -= 1000000000;
     }
-    old_lock_count = lock->lock_count; 
-    old_owner = lock->owner;
-    lock->lock_count = 0;
     rv = pthread_cond_timedwait(&cond->cond, &lock->mutex, &spec);
-    lock->lock_count = old_lock_count;
-    lock->owner = old_owner;
     return rv;
 }
 
@@ -648,14 +624,15 @@ ipmi_posix_thread_free_os_handler(os_handler_t *os_hnd)
 }
 
 void
-ipmi_posix_thread_os_handler_set_sel(os_handler_t *os_hnd, selector_t *sel)
+ipmi_posix_thread_os_handler_set_sel(os_handler_t *os_hnd,
+				     struct selector_s *sel)
 {
     pt_os_hnd_data_t  *info = os_hnd->internal_data;
 
     info->sel = sel;
 }
 
-selector_t *
+struct selector_s *
 ipmi_posix_thread_os_handler_get_sel(os_handler_t *os_hnd)
 {
     pt_os_hnd_data_t  *info = os_hnd->internal_data;
@@ -911,7 +888,7 @@ static os_handler_t ipmi_posix_thread_os_handler =
 };
 
 os_handler_t *
-ipmi_posix_thread_get_os_handler(void)
+ipmi_posix_thread_get_os_handler2(int wake_sig)
 {
     os_handler_t     *rv;
     pt_os_hnd_data_t *info;
@@ -942,9 +919,59 @@ ipmi_posix_thread_get_os_handler(void)
     }
 #endif
 
+    info->wake_sig = wake_sig;
+
     return rv;
 }
 
+os_handler_t *
+ipmi_posix_thread_get_os_handler(void)
+{
+    return ipmi_posix_thread_get_os_handler2(0);
+}
+
+struct sel_lock_s
+{
+    os_handler_t *os_hnd;
+    os_hnd_lock_t *lock;
+};
+
+static sel_lock_t *
+slock_alloc(void *cb_data)
+{
+    os_handler_t *os_hnd = cb_data;
+    sel_lock_t *l;
+
+    l = os_hnd->mem_alloc(sizeof(*l));
+    if (!l)
+	return NULL;
+    l->os_hnd = os_hnd;
+    if (os_hnd->create_lock(os_hnd, &l->lock)) {
+	os_hnd->mem_free(l);
+	l = NULL;
+    }
+    return l;
+}
+
+static void
+slock_free(sel_lock_t *l)
+{
+    l->os_hnd->destroy_lock(l->os_hnd, l->lock);
+    l->os_hnd->mem_free(l);
+}
+
+static void
+slock_lock(sel_lock_t *l)
+{
+    l->os_hnd->lock(l->os_hnd, l->lock);
+}
+
+static void
+slock_unlock(sel_lock_t *l)
+{
+    l->os_hnd->unlock(l->os_hnd, l->lock);
+}
+
 os_handler_t *
 ipmi_posix_thread_setup_os_handler(int wake_sig)
 {
@@ -953,14 +980,15 @@ ipmi_posix_thread_setup_os_handler(int wake_sig)
     struct sigaction act;
     int              rv;
 
-    os_hnd = ipmi_posix_thread_get_os_handler();
+    os_hnd = ipmi_posix_thread_get_os_handler2(wake_sig);
     if (!os_hnd)
 	return NULL;
 
     info = os_hnd->internal_data;
-    info->wake_sig = wake_sig;
 
-    rv = sel_alloc_selector(os_hnd, &info->sel);
+    rv = sel_alloc_selector_thread(&info->sel, wake_sig,
+				   slock_alloc, slock_free,
+				   slock_lock, slock_unlock, os_hnd);
     if (rv) {
 	ipmi_posix_thread_free_os_handler(os_hnd);
 	os_hnd = NULL;
